<?php

namespace Teknisa\FIN\Modules\ContaPagar\Factory;

use Doctrine\ORM\EntityManager;
use Teknisa\Model\Financeiro\Entities\Retornodda;
use Teknisa\Model\Financeiro\Entities\Boletpagar;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\Model\Geral\Entities\Empresa;

class InconsistenciasDDAFactory {

    protected $entityManager;

    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    public function getEmprbanc($cdempresa, $cdbanco, $nrorg){
        $parans = array();
        $parans['cdempresa'] = $cdempresa;
        $parans['cdbanco'] = $cdbanco;
        $parans['nrorg'] = $nrorg;

        $sql = 'SELECT E.CDREDUCONT, E.CDTPBAIXAPAG
                  FROM EMPRBANC E, BANCO B
                 WHERE E.CDEMPRESA  = :CDEMPRESA
                   AND B.CDBANCCNAB = :CDBANCO
                   AND B.NRORG      = :NRORG
                   AND B.CDBANCO    = E.CDBANCO
                   AND B.NRORG      = E.NRORG
                   AND (E.CDREDUCONT IS NOT NULL OR CDTPBAIXAPAG IS NOT NULL)';
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($parans);
        return $std->fetch();
    }
    
    public function getEmpresa($retorno, $nrorg){
        $params = array(
            'nrorg'        => $nrorg,
            'nrinsjurempr' => $retorno->getCnpjEmpresa()
        );
        $empresa = $this->entityManager->getRepository(Empresa::class)->findOneBy($params);
        return $empresa;
    }

    public function getBoletPagar($parans)
    {
        $boletpagar = $this->entityManager->getRepository(Boletpagar::class)->findOneBy($parans);
        return $boletpagar;
    }

    public function getBoletos($parans, $nrorg){
        $paramdda = $this->getParamdda(5, $nrorg);
        
        if($paramdda[0]['IDCONSISTE'] == 'N'){
            $sql = "SELECT BP.CDFILIAL   ,
                           BP.NRLANBOLPAG,
                           BP.CDBARRAPAG ,
                           BP.CDEMPRESA  ,
                           BP.DTVENBOLPAG,
                           BP.CDREDUCONT ,
                           BP.CDFORNECED ,
                           FI.NRINSJURFILI
                    FROM BOLETPAGAR BP
                    INNER JOIN CONTAPAGAR CP ON CP.CDFILIBPAG   = BP.CDFILIAL
                                            AND CP.NRLANBOLPAG  = BP.NRLANBOLPAG
                    INNER JOIN NOTAFISCAL NF ON NF.CDFILIAL     = CP.CDFILIAL
                                            AND NF.NRLANCTONF   = CP.NRLANCTONF
                                            AND ((LPAD(NF.NRNOTAFISC||NF.CDSERIEENTR, 20, 0) = NVL(:NRNOTAFISC, 'XXXXXXXXXXXXXXXXXXXX'))
                                             OR (LPAD(NF.NRNOTAFISC, 20, 0) = NVL(:NRNOTAFISC, 'XXXXXXXXXXXXXXXXXXXX'))
                                             OR (:TNRNOTAFISC = 'T'))
                    INNER JOIN FORNECEDOR FO ON FO.CDFORNECED   = BP.CDFORNECED
                                            AND ((FO.NRINSJURFORN = :CDFORNECED)
                                             OR ((SUBSTR(FO.NRINSJURFORN,1,8) = SUBSTR(:CDFORNECED,1,8)) AND (FO.CDGRUPFORN IS NOT NULL))
                                             OR (:TCDFORNECED = 'T'))
                    INNER JOIN EMPRESA EM    ON EM.CDEMPRESA    = BP.CDEMPRESA
                                            AND EM.NRINSJUREMPR = :CNPJEMPRESA
                    INNER JOIN FILIAL FI     ON FI.CDFILIAL     = NF.CDFILIRECE
                    WHERE ((BP.DTVENBOLPAG = :DTVENBOLPAG)
                       OR ((BP.DTVENBOLPAG BETWEEN TO_DATE(:DTVENBOLPAG)-180 AND TO_DATE(:DTVENBOLPAG)+180) 
                      AND (:TDTVENBOLPAG = 'T')))
                      AND ((BP.IDSTATBPAG = :IDSTATUSA
                       OR BP.IDSTATBPAG   = :IDSTATUSB
                       OR BP.IDSTATBPAG   = :IDSTATUSC)
                       OR (:IDSTATUSA     = 'T'))
                      AND (BP.VRFINALBPAG = :VRFINALBPAG
                       OR (:TODOSVR = 'T'))";
        }else{
            $sql = "SELECT BP.CDFILIAL   ,
                           BP.NRLANBOLPAG,
                           BP.CDBARRAPAG ,
                           BP.CDEMPRESA  ,
                           BP.DTVENBOLPAG,
                           BP.CDREDUCONT ,
                           BP.CDFORNECED ,
                           FI.NRINSJURFILI
                    FROM BOLETPAGAR BP
                    INNER JOIN CONTAPAGAR CP ON CP.CDFILIBPAG   = BP.CDFILIAL
                                            AND CP.NRLANBOLPAG  = BP.NRLANBOLPAG
                    INNER JOIN NOTAFISCAL NF ON NF.CDFILIAL     = CP.CDFILIAL
                                            AND NF.NRLANCTONF   = CP.NRLANCTONF
                                            AND ((LPAD(NF.NRNOTAFISC||NF.CDSERIEENTR, 20, 0) = NVL(:NRNOTAFISC, 'XXXXXXXXXXXXXXXXXXXX'))
                                             OR (LPAD(NF.NRNOTAFISC, 20, 0) = NVL(:NRNOTAFISC, 'XXXXXXXXXXXXXXXXXXXX'))
                                             OR (:TNRNOTAFISC = 'T'))
                    INNER JOIN FORNECEDOR FO ON FO.CDFORNECED   = BP.CDFORNECED
                                            AND ((FO.NRINSJURFORN = :CDFORNECED)
                                             OR ((SUBSTR(FO.NRINSJURFORN,1,8) = SUBSTR(:CDFORNECED,1,8)) AND (FO.CDGRUPFORN IS NOT NULL))
                                             OR (:TCDFORNECED = 'T'))
                    INNER JOIN EMPRESA EM    ON EM.CDEMPRESA    = BP.CDEMPRESA
                                            AND EM.NRINSJUREMPR = :CNPJEMPRESA
                    INNER JOIN FILIAL FI     ON FI.CDFILIAL     = NF.CDFILIRECE
                    WHERE ((BP.DTVENBOLPAG = :DTVENBOLPAG)
                       OR ((BP.DTVENBOLPAG BETWEEN TO_DATE(:DTVENBOLPAG)-180 AND TO_DATE(:DTVENBOLPAG)+180) 
                      AND (:TDTVENBOLPAG = 'T')))
                      AND ((BP.IDSTATBPAG = :IDSTATUSA
                       OR BP.IDSTATBPAG   = :IDSTATUSB
                       OR BP.IDSTATBPAG   = :IDSTATUSC)
                       OR (:IDSTATUSA     = 'T'))
                      AND ((BP.VRFINALBPAG BETWEEN (:VRFINALBPAG-0.1) AND (:VRFINALBPAG+0.1))
                       OR (:TODOSVR                     = 'T'))";
        }
                     
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($parans);
        return $std->fetchAll();
    }

    public function insereRetornoDDA(
        $retorno, $dsincons, $row
    ){
        /** @var Notafiscal $notafiscal */
        $retornodda = new Retornodda();
        $retornodda->setDsempresa($retorno->getCnpjEmpresa());
        $retornodda->setCdempresa($row['CDEMPRESA']);
        $retornodda->setCdbanco($retorno->getCodigoBanco());
        $retornodda->setCdbarra($retorno->getCdbarrapag());
        $retornodda->setCnpjfilial($row['NRINSJURFILI']);
        $retornodda->setCdreducont($row['CDREDUCONT']);
        $retornodda->setStatus($row['STATUS']);
        $retornodda->setCdforneced($retorno->getCdforneced());
        $retornodda->setNmforneced(substr($retorno->getNmforneced(), 30));
        $retornodda->setDtvenbolpag($row['DTVENBOLPAG']);
        $retornodda->setNrnotafisc($retorno->getNrnotafisc());
        $retornodda->setVrfinalbpag($retorno->getVrfinalbpag());
        $retornodda->setDsincons($dsincons);
        $retornodda->setNrorg($row['NRORG']);
        $retornodda->setNrorginclusao($row['NRORG']);
        $retornodda->setDtinclusao(DateUtil::getDataAtual());
        return $retornodda;
    }
    
    public function getParamdda($nrseqparam, $nrorg){
        $parans = array();
        $parans['nrseqparam'] = $nrseqparam;
        $parans['nrorg']      = $nrorg;

        $sql = 'SELECT P.NRORG, P.NRSEQPARAM, P.DSVALIDADDA, P.IDREMOVEARQ, P.IDCONSISTE
                  FROM PARAMDDA P
                 WHERE P.NRSEQPARAM  = :NRSEQPARAM
                   AND P.NRORG       = :NRORG';
                   
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($parans);
        $fetchAll = $std->fetchAll();
        
        if(!$fetchAll)
            throw new \Exception('Operação bloqueada. Parametrização DDA não encontrada ou faltando informação.');
        
        return $fetchAll;
    }
}