<?php

namespace Teknisa\FIN\Modules\ContaReceber\Factory;

use Teknisa\Libs\Util\DateUtil;
use Doctrine\ORM\EntityManager;
use Financeiro\Entities\Ratctreceber;

/**
 * Class RateioContaFactory
 * @package Teknisa\FIN\Modules\ContaReceber\Factory
 */
class RateioContaFactory {

    /**
     * @param EntityManager $entityManager
     */
    public function __construct(EntityManager $entityManager) {
        $this->entityManager = $entityManager;
    }

    /**
     * Busca Rateio Conta a Receber
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanctonf
     * @param $nrseqvenrec
     * @param $nrparcelarec
     * @param $cdcentcust
     * @return Ratctreceber
     */
    public static function buscaRateioConta(
        $nrorg, $cdfilial, $nrlanctonf, $nrseqvenrec, $nrparcelarec, $cdcentcust
    ){
        /** @var Ratctreceber $ratctreceber */
        $ratctreceber = Ratctreceber::findOneByNrorgAndCdfilialAndNrlanctonfAndNrseqvenrecAndNrparcelarecAndCdcentcust(
            $nrorg, $cdfilial, $nrlanctonf, $nrseqvenrec, $nrparcelarec, $cdcentcust
        );

        return $ratctreceber;
    }

    /**
     * Busca Rateios Conta a Receber
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanctonf
     * @param $nrseqvenrec
     * @param $nrparcelarec
     * @return Ratctreceber
     */
    public static function buscaRateiosConta(
        $nrorg, $cdfilial, $nrlanctonf, $nrseqvenrec, $nrparcelarec
    ){
        /** @var Ratctreceber $ratctreceber */
        $ratctreceber = Ratctreceber::findByNrorgAndCdfilialAndNrlanctonfAndNrseqvenrecAndNrparcelarec(
            $nrorg, $cdfilial, $nrlanctonf, $nrseqvenrec, $nrparcelarec
        );

        return $ratctreceber;
    }


    /**
     * Busca Centro de Custo do Maior Valor Rateio Conta a Receber
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanctonf
     * @param $nrseqvenrec
     * @param $nrparcelarec
     * @param $cdfiratctrec
     * @return bool
     */
    public function buscaCentroCustoMaiorValorRateioConta(
        $nrorg, $cdfilial, $nrlanctonf, $nrseqvenrec, $nrparcelarec, $cdfiratctrec
    ) {
        $connection = $this->entityManager->getConnection();

        $params                   = array();
        $params['P_NRORG']        = $nrorg;
        $params['P_CDFILIAL']     = $cdfilial;
        $params['P_NRLANCTONF']   = $nrlanctonf;
        $params['P_NRSEQVENREC']  = $nrseqvenrec;
        $params['P_NRPARCELAREC'] = $nrparcelarec;
        $params['P_CDFIRATCTREC'] = $cdfiratctrec;

        $sql = "  SELECT MIN(CDCENTCUST) CDCENTCUST
                    FROM RATCTRECEBER
                    WHERE NRORG        = :P_NRORG
                      AND CDFILIAL     = :P_CDFILIAL
                      AND NRLANCTONF   = :P_NRLANCTONF
                      AND NRSEQVENREC  = :P_NRSEQVENREC
                      AND NRPARCELAREC = :P_NRPARCELAREC
                      AND CDFIRATCTREC = :P_CDFIRATCTREC
                      AND VRRATCCCTREC = (SELECT MAX(VRRATCCCTREC) VRRATCCCTREC
                                                FROM RATCTRECEBER
                                               WHERE NRORG        = :P_NRORG
                                                 AND CDFILIAL     = :P_CDFILIAL
                                                 AND NRLANCTONF   = :P_NRLANCTONF
                                                 AND NRSEQVENREC  = :P_NRSEQVENREC
                                                 AND NRPARCELAREC = :P_NRPARCELAREC
                                                 AND CDFIRATCTREC = :P_CDFIRATCTREC)";

        $std = $connection->prepare($sql);
        $std->execute($params);

        return $std->fetchAll();
    }


    /**
     * Insere Rateio Conta a Receber
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanctonf
     * @param $nrseqvenrec
     * @param $nrparcelarec
     * @param $cdcentcust
     * @param $vrratccctrec
     * @return Ratctreceber
     */
    public static function insereRateioConta(
        $nrorg, $cdfilial, $nrlanctonf, $nrseqvenrec, $nrparcelarec, $cdfiratctrec, $cdcentcust, $vrratccctrec
    ){
        /** @var Ratctreceber $ratctreceber */
        $ratctreceber = New Ratctreceber();
        $ratctreceber->setNrorg($nrorg);
        $ratctreceber->setCdfilial($cdfilial);
        $ratctreceber->setNrlanctonf($nrlanctonf);
        $ratctreceber->setNrseqvenrec($nrseqvenrec);
        $ratctreceber->setNrparcelarec($nrparcelarec);
        $ratctreceber->setCdfiratctrec($cdfiratctrec);
        $ratctreceber->setCdcentcust($cdcentcust);
        $ratctreceber->setVrratccctrec($vrratccctrec);
        $ratctreceber->setDtinclusao(DateUtil::getDataAtual());

        return $ratctreceber;
    }

    /**
     * Atualiza Rateio Conta Receber
     * @param Ratctreceber $ratctreceber
     * @param $vrratccctrec
     * @return Ratctreceber
     */
    public static function atualizaMaiorValorRateioConta(
        Ratctreceber $ratctreceber, $vrratccctrec
    ){
        $vrratccctrec += $ratctreceber->getVrratccctrec();
        $ratctreceber->setVrratccctrec($vrratccctrec);

        return $ratctreceber;
    }

}